@@ -128,7 +128,7 @@ static void at_ccfc_query(struct ofono_call_forwarding *cf, int type, int cls,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[64];
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	cbd->user = GINT_TO_POINTER(cls);
@@ -143,8 +143,7 @@ static void at_ccfc_query(struct ofono_call_forwarding *cf, int type, int cls,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, 0, NULL, data);
 }
@@ -166,7 +165,7 @@ static void at_ccfc_set(struct ofono_call_forwarding *cf, const char *buf,
 	GAtChat *chat = ofono_call_forwarding_get_data(cf);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	if (g_at_chat_send(chat, buf, none_prefix,
@@ -174,8 +173,7 @@ static void at_ccfc_set(struct ofono_call_forwarding *cf, const char *buf,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
 }
@@ -259,7 +257,7 @@ static int at_ccfc_probe(struct ofono_call_forwarding *cf, unsigned int vendor,
 {
 	GAtChat *chat = data;
 
-	ofono_call_forwarding_set_data(cf, chat);
+	ofono_call_forwarding_set_data(cf, g_at_chat_clone(chat));
 	g_idle_add(at_ccfc_register, cf);
 
 	return 0;
@@ -267,6 +265,10 @@ static int at_ccfc_probe(struct ofono_call_forwarding *cf, unsigned int vendor,
 
 static void at_ccfc_remove(struct ofono_call_forwarding *cf)
 {
+	GAtChat *chat = ofono_call_forwarding_get_data(cf);
+
+	g_at_chat_unref(chat);
+	ofono_call_forwarding_set_data(cf, NULL);
 }
 
 static struct ofono_call_forwarding_driver driver = {
