@@ -26,6 +26,7 @@
 #define _GNU_SOURCE
 #include <string.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <stdint.h>
 
 #include <glib.h>
@@ -75,6 +76,9 @@ struct ofono_stk {
 	struct stk_icon_id idle_mode_icon;
 	struct timeval get_inkey_start_ts;
 	int dtmf_id;
+
+	__ofono_sms_sim_download_cb_t sms_pp_cb;
+	void *sms_pp_userdata;
 };
 
 struct envelope_op {
@@ -113,7 +117,7 @@ static int stk_respond(struct ofono_stk *stk, struct stk_response *rsp,
 	rsp->qualifier = stk->pending_cmd->qualifier;
 
 	tlv = stk_pdu_from_response(rsp, &tlv_len);
-	if (!tlv)
+	if (tlv == NULL)
 		return -EINVAL;
 
 	stk_command_free(stk->pending_cmd);
@@ -205,7 +209,7 @@ static int stk_send_envelope(struct ofono_stk *stk, struct stk_envelope *e,
 
 	e->dst = STK_DEVICE_IDENTITY_TYPE_UICC;
 	tlv = stk_pdu_from_envelope(e, &tlv_len);
-	if (!tlv)
+	if (tlv == NULL)
 		return -EINVAL;
 
 	op = g_new0(struct envelope_op, 1);
@@ -257,6 +261,42 @@ void __ofono_cbs_sim_download(struct ofono_stk *stk, const struct cbs *msg)
 		stk_cbs_download_cb(stk, FALSE, NULL, -1);
 }
 
+static void stk_sms_download_cb(struct ofono_stk *stk, gboolean ok,
+				const unsigned char *data, int len)
+{
+	DBG("SMS-PP download to UICC reported %s", ok ? "success" : "error");
+
+	if (stk->sms_pp_cb)
+		stk->sms_pp_cb(ok, data, len, stk->sms_pp_userdata);
+}
+
+int __ofono_sms_sim_download(struct ofono_stk *stk, const struct sms *msg,
+				__ofono_sms_sim_download_cb_t cb, void *data)
+{
+	struct stk_envelope e;
+
+	if (msg->type != SMS_TYPE_DELIVER)
+		return -EINVAL;
+
+	DBG("");
+
+	memset(&e, 0, sizeof(e));
+
+	e.type = STK_ENVELOPE_TYPE_SMS_PP_DOWNLOAD;
+	e.src = STK_DEVICE_IDENTITY_TYPE_NETWORK;
+
+	e.sms_pp_download.address.number = (char *) msg->sc_addr.address;
+	e.sms_pp_download.address.ton_npi = msg->sc_addr.numbering_plan |
+		(msg->sc_addr.number_type << 4);
+	memcpy(&e.sms_pp_download.message, &msg->deliver, sizeof(msg->deliver));
+
+	stk->sms_pp_cb = cb;
+	stk->sms_pp_userdata = data;
+
+	return stk_send_envelope(stk, &e, stk_sms_download_cb,
+					ENVELOPE_RETRIES_DEFAULT);
+}
+
 static char *dbus_apply_text_attributes(const char *text,
 					const struct stk_text_attribute *attr)
 {
@@ -264,6 +304,9 @@ static char *dbus_apply_text_attributes(const char *text,
 	const uint8_t *j = attr->attributes;
 	const uint8_t *end = j + attr->len;
 
+	if (text == NULL)
+		return NULL;
+
 	if (attr->len & 3)
 		return NULL;
 
@@ -300,7 +343,7 @@ static struct stk_menu *stk_menu_create(const char *title,
 
 	ret->title = dbus_apply_text_attributes(title ? title : "",
 						title_attr);
-	if (!ret->title)
+	if (ret->title == NULL)
 		ret->title = g_strdup(title ? title : "");
 
 	memcpy(&ret->icon, icon, sizeof(ret->icon));
@@ -324,7 +367,7 @@ static struct stk_menu *stk_menu_create(const char *title,
 			text = dbus_apply_text_attributes(item->text, &attr);
 		}
 
-		if (!text)
+		if (text == NULL)
 			text = strdup(item->text);
 
 		ret->items[i].text = text;
@@ -409,7 +452,7 @@ static void emit_menu_changed(struct ofono_stk *stk)
 
 	signal = dbus_message_new_signal(path, OFONO_STK_INTERFACE,
 						"PropertyChanged");
-	if (!signal) {
+	if (signal == NULL) {
 		ofono_error("Unable to allocate new %s.PropertyChanged signal",
 				OFONO_SIM_APP_INTERFACE);
 
@@ -429,12 +472,17 @@ static void stk_alpha_id_set(struct ofono_stk *stk,
 		const char *text, const struct stk_text_attribute *attr,
 		const struct stk_icon_id *icon)
 {
-	/* TODO */
+	char *alpha = dbus_apply_text_attributes(text, attr);
+
+	if (alpha || icon)
+		stk_agent_display_action_info(stk->current_agent, alpha, icon);
+
+	g_free(alpha);
 }
 
 static void stk_alpha_id_unset(struct ofono_stk *stk)
 {
-	/* TODO */
+	stk_agent_request_cancel(stk->current_agent);
 }
 
 static int duration_to_msecs(const struct stk_duration *duration)
@@ -468,7 +516,7 @@ static DBusMessage *stk_get_properties(DBusConnection *conn,
 	unsigned char icon;
 
 	reply = dbus_message_new_method_return(msg);
-	if (!reply)
+	if (reply == NULL)
 		return NULL;
 
 	dbus_message_iter_init_append(reply, &iter);
@@ -477,7 +525,7 @@ static DBusMessage *stk_get_properties(DBusConnection *conn,
 					OFONO_PROPERTIES_ARRAY_SIGNATURE,
 					&dict);
 
-	str = stk->idle_mode_text ? stk->idle_mode_text : "";
+	str = stk->idle_mode_text;
 	ofono_dbus_dict_append(&dict, "IdleModeText", DBUS_TYPE_STRING, &str);
 
 	icon = stk->idle_mode_icon.id;
@@ -590,13 +638,13 @@ static DBusMessage *stk_register_agent(DBusConnection *conn,
 	stk->default_agent = stk_agent_new(agent_path,
 						dbus_message_get_sender(msg),
 						FALSE);
-	if (!stk->default_agent)
+	if (stk->default_agent == NULL)
 		return __ofono_error_failed(msg);
 
 	stk_agent_set_removed_notify(stk->default_agent,
 					default_agent_notify, stk);
 
-	if (!stk->session_agent)
+	if (stk->session_agent == NULL)
 		stk->current_agent = stk->default_agent;
 
 	return dbus_message_new_method_return(msg);
@@ -614,7 +662,7 @@ static DBusMessage *stk_unregister_agent(DBusConnection *conn,
 					DBUS_TYPE_INVALID) == FALSE)
 		return __ofono_error_invalid_args(msg);
 
-	if (!stk->default_agent)
+	if (stk->default_agent == NULL)
 		return __ofono_error_failed(msg);
 
 	if (!stk_agent_matches(stk->default_agent, agent_path, agent_bus))
@@ -656,7 +704,7 @@ static void menu_selection_envelope_cb(struct ofono_stk *stk, gboolean ok,
 	stk->session_agent = stk_agent_new(agent_path,
 					dbus_message_get_sender(stk->pending),
 					TRUE);
-	if (!stk->session_agent) {
+	if (stk->session_agent == NULL) {
 		reply = __ofono_error_failed(stk->pending);
 
 		goto out;
@@ -687,7 +735,7 @@ static DBusMessage *stk_select_item(DBusConnection *conn,
 	if (stk->pending || stk->session_agent)
 		return __ofono_error_busy(msg);
 
-	if (!menu)
+	if (menu == NULL)
 		return __ofono_error_not_supported(msg);
 
 	if (dbus_message_get_args(msg, NULL,
@@ -796,7 +844,7 @@ static gboolean handle_command_send_sms(const struct stk_command *cmd,
 
 	sms_atom = __ofono_modem_find_atom(modem, OFONO_ATOM_TYPE_SMS);
 
-	if (!sms_atom || !__ofono_atom_get_registered(sms_atom)) {
+	if (sms_atom == NULL || !__ofono_atom_get_registered(sms_atom)) {
 		rsp->result.type = STK_RESULT_TYPE_NOT_CAPABLE;
 		return TRUE;
 	}
@@ -830,17 +878,15 @@ static gboolean handle_command_set_idle_text(const struct stk_command *cmd,
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
 	const char *path = __ofono_atom_get_path(stk->atom);
-	char *idle_mode_text = NULL;
+	char *idle_mode_text;
 
-	if (cmd->setup_idle_mode_text.text) {
-		idle_mode_text = dbus_apply_text_attributes(
+	idle_mode_text = dbus_apply_text_attributes(
 					cmd->setup_idle_mode_text.text,
 					&cmd->setup_idle_mode_text.text_attr);
 
-		if (!idle_mode_text) {
-			rsp->result.type = STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD;
-			return TRUE;
-		}
+	if (idle_mode_text == NULL) {
+		rsp->result.type = STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD;
+		return TRUE;
 	}
 
 	if (stk->idle_mode_text)
@@ -848,7 +894,6 @@ static gboolean handle_command_set_idle_text(const struct stk_command *cmd,
 
 	stk->idle_mode_text = idle_mode_text;
 
-	idle_mode_text = idle_mode_text ? idle_mode_text : "";
 	ofono_dbus_signal_property_changed(conn, path, OFONO_STK_INTERFACE,
 						"IdleModeText",
 						DBUS_TYPE_STRING,
@@ -1126,7 +1171,7 @@ static gboolean handle_command_select_item(const struct stk_command *cmd,
 {
 	stk->select_item_menu = stk_menu_create_from_select_item(cmd);
 
-	if (!stk->select_item_menu) {
+	if (stk->select_item_menu == NULL) {
 		rsp->result.type = STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD;
 
 		return TRUE;
@@ -1161,6 +1206,9 @@ static void display_text_cb(enum stk_agent_result result, void *user_data)
 {
 	struct ofono_stk *stk = user_data;
 	gboolean confirm;
+	struct stk_response rsp;
+	static unsigned char screen_busy_result[] = { 0x01 };
+	static struct ofono_error error = { .type = OFONO_ERROR_TYPE_FAILURE };
 
 	stk->respond_on_exit = FALSE;
 
@@ -1208,6 +1256,15 @@ static void display_text_cb(enum stk_agent_result result, void *user_data)
 			STK_RESULT_TYPE_NO_RESPONSE : STK_RESULT_TYPE_SUCCESS);
 		break;
 
+	case STK_AGENT_RESULT_BUSY:
+		memset(&rsp, 0, sizeof(rsp));
+		rsp.result.type = STK_RESULT_TYPE_TERMINAL_BUSY;
+		rsp.result.additional_len = sizeof(screen_busy_result);
+		rsp.result.additional = screen_busy_result;
+		if (stk_respond(stk, &rsp, stk_command_cb))
+			stk_command_cb(&error, stk);
+		break;
+
 	case STK_AGENT_RESULT_TERMINATE:
 	default:
 		send_simple_response(stk, STK_RESULT_TYPE_USER_TERMINATED);
@@ -1226,11 +1283,14 @@ static gboolean handle_command_display_text(const struct stk_command *cmd,
 	char *text = dbus_apply_text_attributes(dt->text, &dt->text_attr);
 	int err;
 
-	if (!text) {
+	if (text == NULL) {
 		rsp->result.type = STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD;
 		return TRUE;
 	}
 
+	if (qualifier & (1 << 7))
+		timeout = stk->timeout * 1000;
+
 	if (dt->duration.interval)
 		timeout = duration_to_msecs(&dt->duration);
 
@@ -1321,6 +1381,7 @@ static void request_confirmation_cb(enum stk_agent_result result,
 		break;
 
 	case STK_AGENT_RESULT_TERMINATE:
+	default:
 		send_simple_response(stk, STK_RESULT_TYPE_USER_TERMINATED);
 		break;
 	}
@@ -1363,6 +1424,7 @@ static void request_key_cb(enum stk_agent_result result, char *string,
 		break;
 
 	case STK_AGENT_RESULT_TERMINATE:
+	default:
 		send_simple_response(stk, STK_RESULT_TYPE_USER_TERMINATED);
 		break;
 	}
@@ -1385,7 +1447,7 @@ static gboolean handle_command_get_inkey(const struct stk_command *cmd,
 	 */
 	int err;
 
-	if (!text) {
+	if (text == NULL) {
 		rsp->result.type = STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD;
 		return TRUE;
 	}
@@ -1460,6 +1522,7 @@ static void request_string_cb(enum stk_agent_result result, char *string,
 		break;
 
 	case STK_AGENT_RESULT_TERMINATE:
+	default:
 		send_simple_response(stk, STK_RESULT_TYPE_USER_TERMINATED);
 		break;
 	}
@@ -1478,7 +1541,7 @@ static gboolean handle_command_get_input(const struct stk_command *cmd,
 	gboolean hidden = (qualifier & (1 << 2)) != 0;
 	int err;
 
-	if (!text) {
+	if (text == NULL) {
 		rsp->result.type = STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD;
 		return TRUE;
 	}
@@ -1522,7 +1585,7 @@ static void call_setup_connected(struct ofono_call *call, void *data)
 	static struct ofono_error error = { .type = OFONO_ERROR_TYPE_FAILURE };
 	static unsigned char facility_rejected_result[] = { 0x9d };
 
-	if (!call || call->status == CALL_STATUS_DISCONNECTED) {
+	if (call == NULL || call->status == CALL_STATUS_DISCONNECTED) {
 		memset(&rsp, 0, sizeof(rsp));
 
 		rsp.result.type = STK_RESULT_TYPE_NETWORK_UNAVAILABLE;
@@ -1548,7 +1611,7 @@ static void call_setup_cancel(struct ofono_stk *stk)
 
 	vc_atom = __ofono_modem_find_atom(__ofono_atom_get_modem(stk->atom),
 						OFONO_ATOM_TYPE_VOICECALL);
-	if (!vc_atom)
+	if (vc_atom == NULL)
 		return;
 
 	vc = __ofono_atom_get_data(vc_atom);
@@ -1596,7 +1659,7 @@ static void confirm_call_cb(enum stk_agent_result result, gboolean confirm,
 	if (vc_atom)
 		vc = __ofono_atom_get_data(vc_atom);
 
-	if (!vc) {
+	if (vc == NULL) {
 		send_simple_response(stk, STK_RESULT_TYPE_NOT_CAPABLE);
 		return;
 	}
@@ -1604,7 +1667,7 @@ static void confirm_call_cb(enum stk_agent_result result, gboolean confirm,
 	if (sc->alpha_id_call_setup) {
 		alpha_id = dbus_apply_text_attributes(sc->alpha_id_call_setup,
 						&sc->text_attr_call_setup);
-		if (!alpha_id) {
+		if (alpha_id == NULL) {
 			send_simple_response(stk,
 					STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD);
 			return;
@@ -1683,7 +1746,7 @@ static gboolean handle_command_set_up_call(const struct stk_command *cmd,
 	if (vc_atom)
 		vc = __ofono_atom_get_data(vc_atom);
 
-	if (!vc) {
+	if (vc == NULL) {
 		rsp->result.type = STK_RESULT_TYPE_NOT_CAPABLE;
 		return TRUE;
 	}
@@ -1695,13 +1758,12 @@ static gboolean handle_command_set_up_call(const struct stk_command *cmd,
 		return TRUE;
 	}
 
-	if (sc->alpha_id_usr_cfm) {
-		alpha_id = dbus_apply_text_attributes(sc->alpha_id_usr_cfm,
-							&sc->text_attr_usr_cfm);
-		if (!alpha_id) {
-			rsp->result.type = STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD;
-			return TRUE;
-		}
+	alpha_id = dbus_apply_text_attributes(sc->alpha_id_usr_cfm ?
+						sc->alpha_id_usr_cfm : "",
+						&sc->text_attr_usr_cfm);
+	if (alpha_id == NULL) {
+		rsp->result.type = STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD;
+		return TRUE;
 	}
 
 	err = stk_agent_confirm_call(stk->current_agent, alpha_id,
@@ -1731,7 +1793,7 @@ static void send_ussd_cancel(struct ofono_stk *stk)
 
 	atom = __ofono_modem_find_atom(__ofono_atom_get_modem(stk->atom),
 						OFONO_ATOM_TYPE_USSD);
-	if (!atom)
+	if (atom == NULL)
 		return;
 
 	ussd = __ofono_atom_get_data(atom);
@@ -1846,7 +1908,7 @@ static gboolean handle_command_send_ussd(const struct stk_command *cmd,
 	}
 
 	atom = __ofono_modem_find_atom(modem, OFONO_ATOM_TYPE_USSD);
-	if (!atom || !__ofono_atom_get_registered(atom)) {
+	if (atom == NULL || !__ofono_atom_get_registered(atom)) {
 		rsp->result.type = STK_RESULT_TYPE_NOT_CAPABLE;
 		return TRUE;
 	}
@@ -1950,6 +2012,77 @@ static gboolean handle_command_refresh(const struct stk_command *cmd,
 	return TRUE;
 }
 
+static void get_time(struct stk_response *rsp)
+{
+	time_t now;
+	struct tm *t;
+
+	time(&now);
+	t = localtime(&now);
+
+	rsp->result.type = STK_RESULT_TYPE_SUCCESS;
+
+	if (t->tm_year > 100)
+		rsp->provide_local_info.datetime.year = t->tm_year - 100;
+	else
+		rsp->provide_local_info.datetime.year = t->tm_year;
+
+	rsp->provide_local_info.datetime.month = t->tm_mon + 1;
+	rsp->provide_local_info.datetime.day = t->tm_mday;
+	rsp->provide_local_info.datetime.hour = t->tm_hour;
+	rsp->provide_local_info.datetime.minute = t->tm_min;
+	rsp->provide_local_info.datetime.second = t->tm_sec;
+	rsp->provide_local_info.datetime.timezone = t->tm_gmtoff / 900;
+	rsp->provide_local_info.datetime.has_timezone = TRUE;
+
+	return;
+}
+
+static void get_lang(struct stk_response *rsp, struct ofono_stk *stk)
+{
+	char *l;
+	char lang[3];
+	struct ofono_error failure = { .type = OFONO_ERROR_TYPE_FAILURE };
+
+	l = getenv("LANG");
+	if (l == NULL) {
+		rsp->result.type = STK_RESULT_TYPE_NOT_CAPABLE;
+		goto out;
+	}
+
+	memcpy(lang, l, 2);
+	lang[2] = '\0';
+
+	rsp->result.type = STK_RESULT_TYPE_SUCCESS;
+	rsp->provide_local_info.language = lang;
+
+out:
+	if (stk_respond(stk, rsp, stk_command_cb))
+		stk_command_cb(&failure, stk);
+}
+
+static gboolean handle_command_provide_local_info(const struct stk_command *cmd,
+				struct stk_response *rsp, struct ofono_stk *stk)
+{
+	switch (cmd->qualifier) {
+	case 3:
+		DBG("Date, time and time zone");
+		get_time(rsp);
+		return TRUE;
+
+	case 4:
+		DBG("Language setting");
+		get_lang(rsp, stk);
+		return FALSE;
+
+	default:
+		ofono_info("Unsupported Provide Local Info qualifier: %d",
+				cmd->qualifier);
+		rsp->result.type = STK_RESULT_TYPE_NOT_CAPABLE;
+		return TRUE;
+	}
+}
+
 static void send_dtmf_cancel(struct ofono_stk *stk)
 {
 	struct ofono_voicecall *vc = NULL;
@@ -2017,7 +2150,7 @@ static gboolean handle_command_send_dtmf(const struct stk_command *cmd,
 	if (vc_atom)
 		vc = __ofono_atom_get_data(vc_atom);
 
-	if (!vc) {
+	if (vc == NULL) {
 		rsp->result.type = STK_RESULT_TYPE_NOT_CAPABLE;
 		return TRUE;
 	}
@@ -2025,7 +2158,7 @@ static gboolean handle_command_send_dtmf(const struct stk_command *cmd,
 	/* Convert the DTMF string to phone number format */
 	for (pos = 0; cmd->send_dtmf.dtmf[pos] != '\0'; pos++) {
 		digit = strchr(dtmf_from, cmd->send_dtmf.dtmf[pos]);
-		if (!digit) {
+		if (digit == NULL) {
 			rsp->result.type = STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD;
 			return TRUE;
 		}
@@ -2160,7 +2293,7 @@ static gboolean handle_command_play_tone(const struct stk_command *cmd,
 	int err;
 
 	if (pt->tone > sizeof(tone_infos) / sizeof(*tone_infos) ||
-			!tone_infos[pt->tone].name) {
+			tone_infos[pt->tone].name == NULL) {
 		rsp->result.type = STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD;
 
 		return TRUE;
@@ -2168,7 +2301,7 @@ static gboolean handle_command_play_tone(const struct stk_command *cmd,
 
 	text = dbus_apply_text_attributes(pt->alpha_id ? pt->alpha_id : "",
 						&pt->text_attr);
-	if (!text) {
+	if (text == NULL) {
 		rsp->result.type = STK_RESULT_TYPE_DATA_NOT_UNDERSTOOD;
 
 		return TRUE;
@@ -2254,7 +2387,7 @@ void ofono_stk_proactive_command_notify(struct ofono_stk *stk,
 	stk_proactive_command_cancel(stk);
 
 	stk->pending_cmd = stk_command_new_from_pdu(pdu, length);
-	if (!stk->pending_cmd) {
+	if (stk->pending_cmd == NULL) {
 		ofono_error("Can't parse proactive command");
 
 		/*
@@ -2382,6 +2515,11 @@ void ofono_stk_proactive_command_notify(struct ofono_stk *stk,
 							&rsp, stk);
 		break;
 
+	case STK_COMMAND_TYPE_PROVIDE_LOCAL_INFO:
+		respond = handle_command_provide_local_info(stk->pending_cmd,
+								&rsp, stk);
+		break;
+
 	case STK_COMMAND_TYPE_SEND_DTMF:
 		respond = handle_command_send_dtmf(stk->pending_cmd,
 							&rsp, stk);
@@ -2427,7 +2565,7 @@ void ofono_stk_proactive_command_handled_notify(struct ofono_stk *stk,
 
 	cmd = stk_command_new_from_pdu(pdu, length);
 
-	if (!cmd || cmd->status != STK_PARSE_RESULT_OK) {
+	if (cmd == NULL || cmd->status != STK_PARSE_RESULT_OK) {
 		ofono_error("Can't parse proactive command");
 
 		if (cmd)
@@ -2487,7 +2625,7 @@ int ofono_stk_driver_register(const struct ofono_stk_driver *d)
 	if (d->probe == NULL)
 		return -EINVAL;
 
-	g_drivers = g_slist_prepend(g_drivers, (void *)d);
+	g_drivers = g_slist_prepend(g_drivers, (void *) d);
 
 	return 0;
 }
@@ -2496,7 +2634,7 @@ void ofono_stk_driver_unregister(const struct ofono_stk_driver *d)
 {
 	DBG("driver: %p, name: %s", d, d->name);
 
-	g_drivers = g_slist_remove(g_drivers, (void *)d);
+	g_drivers = g_slist_remove(g_drivers, (void *) d);
 }
 
 static void stk_unregister(struct ofono_atom *atom)
@@ -2518,10 +2656,8 @@ static void stk_unregister(struct ofono_atom *atom)
 		stk->cancel_cmd = NULL;
 	}
 
-	if (stk->idle_mode_text) {
-		g_free(stk->idle_mode_text);
-		stk->idle_mode_text = NULL;
-	}
+	g_free(stk->idle_mode_text);
+	stk->idle_mode_text = NULL;
 
 	if (stk->timers_source) {
 		g_source_remove(stk->timers_source);
