@@ -1,21 +1,21 @@
 /*
- * oFono - GSM Telephony Stack for Linux
  *
- * Copyright (C) 2008-2010 Intel Corporation.  All rights reserved.
+ *  oFono - Open Source Telephony
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -471,6 +471,219 @@ static void test_submit_encode()
 	g_free(encoded_pdu);
 }
 
+struct sms_charset_data {
+	char *pdu;
+	int data_len;
+	enum gsm_dialect locking_lang;
+	enum gsm_dialect single_lang;
+	char expected_text[];
+};
+
+static struct sms_charset_data sms_charset_default = {
+	.pdu =
+		"0001000B91" "5310101010" "1000008080" "8060402818" "0E888462C1"
+		"68381E9088" "6442A9582E" "988C06C4E9" "783EA09068" "442A994EA8"
+		"946AC56AB9" "5EB0986C46" "ABD96EB89C" "6EC7EBF97E" "C0A070482C"
+		"1A8FC8A472" "C96C3A9FD0" "A8744AAD5A" "AFD8AC76CB" "ED7ABFE0B0"
+		"784C2E9BCF" "E8B47ACD6E" "BBDFF0B87C" "4EAFDBEFF8" "BC7ECFEFFB"
+		"FF",
+	.data_len = 112,
+	.expected_text = {
+		0x40, 0xc2, 0xa3, 0x24, 0xc2, 0xa5, 0xc3, 0xa8, 0xc3, 0xa9,
+		0xc3, 0xb9, 0xc3, 0xac, 0xc3, 0xb2, 0xc3, 0x87, 0x0a, 0xc3,
+		0x98, 0xc3, 0xb8, 0x0d, 0xc3, 0x85, 0xc3, 0xa5, 0xce, 0x94,
+		0x5f, 0xce, 0xa6, 0xce, 0x93, 0xce, 0x9b, 0xce, 0xa9, 0xce,
+		0xa0, 0xce, 0xa8, 0xce, 0xa3, 0xce, 0x98, 0xce, 0x9e, 0x20,
+		0xc3, 0x86, 0xc3, 0xa6, 0xc3, 0x9f, 0xc3, 0x89, 0x20, 0x21,
+		0x22, 0x23, 0xc2, 0xa4, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a,
+		0x2b, 0x2c, 0x2d, 0x2e, 0x2f, 0x30, 0x31, 0x32, 0x33, 0x34,
+		0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e,
+		0x3f, 0xc2, 0xa1, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47,
+		0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f, 0x50, 0x51,
+		0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0xc3,
+		0x84, 0xc3, 0x96, 0xc3, 0x91, 0xc3, 0x9c, 0xc2, 0xa7, 0xc2,
+		0xbf, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69,
+		0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f, 0x70, 0x71, 0x72, 0x73,
+		0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7a, 0xc3, 0xa4, 0xc3,
+		0xb6, 0xc3, 0xb1, 0xc3, 0xbc, 0xc3, 0xa0, 0x00
+	}
+};
+
+static struct sms_charset_data sms_charset_default_ext = {
+	.pdu =
+		"0001000B91" "5310101010" "100000151B" "C58602DAA0" "36A9CD6BC3"
+		"DBF436BE0D" "705306",
+	.data_len = 19,
+	.expected_text = {
+		0x0c, 0x5e, 0x20, 0x7b,	0x7d, 0x5c, 0x5b, 0x7e,	0x5d, 0x7c,
+		0xe2, 0x82, 0xac, 0x00
+	}
+};
+
+static struct sms_charset_data sms_charset_turkey = {
+	.pdu =
+		"0001000B91" "5310101010" "1000008080" "8060402818" "0E888462C1"
+		"68381E9088" "6442A9582E" "988C06C4E9" "783EA09068" "442A994EA8"
+		"946AC56AB9" "5EB0986C46" "ABD96EB89C" "6EC7EBF97E" "C0A070482C"
+		"1A8FC8A472" "C96C3A9FD0" "A8744AAD5A" "AFD8AC76CB" "ED7ABFE0B0"
+		"784C2E9BCF" "E8B47ACD6E" "BBDFF0B87C" "4EAFDBEFF8" "BC7ECFEFFB"
+		"FF",
+	.data_len = 112,
+	.locking_lang = GSM_DIALECT_TURKISH,
+	.expected_text = {
+		0x40, 0xc2, 0xa3, 0x24, 0xc2, 0xa5, 0xe2, 0x82, 0xac, 0xc3,
+		0xa9, 0xc3, 0xb9, 0xc4, 0xb1, 0xc3, 0xb2, 0xc3, 0x87, 0x0a,
+		0xc4, 0x9e, 0xc4, 0x9f, 0x0d, 0xc3, 0x85, 0xc3, 0xa5, 0xce,
+		0x94, 0x5f, 0xce, 0xa6, 0xce, 0x93, 0xce, 0x9b, 0xce, 0xa9,
+		0xce, 0xa0, 0xce, 0xa8, 0xce, 0xa3, 0xce, 0x98, 0xce, 0x9e,
+		0x20, 0xc5, 0x9e, 0xc5, 0x9f, 0xc3, 0x9f, 0xc3, 0x89, 0x20,
+		0x21, 0x22, 0x23, 0xc2, 0xa4, 0x25, 0x26, 0x27, 0x28, 0x29,
+		0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f, 0x30, 0x31, 0x32, 0x33,
+		0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d,
+		0x3e, 0x3f, 0xc4, 0xb0, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46,
+		0x47, 0x48, 0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f, 0x50,
+		0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5a,
+		0xc3, 0x84, 0xc3, 0x96, 0xc3, 0x91, 0xc3, 0x9c, 0xc2, 0xa7,
+		0xc3, 0xa7, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68,
+		0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f, 0x70, 0x71, 0x72,
+		0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7a, 0xc3, 0xa4,
+		0xc3, 0xb6, 0xc3, 0xb1, 0xc3, 0xbc, 0xc3, 0xa0, 0x00
+	}
+};
+
+static struct sms_charset_data sms_charset_turkey_ext = {
+	.pdu =
+		"0001000B91" "5310101010" "1000001A1B" "C586B2416D" "529BD786B7"
+		"E96D7C1BE0" "02C8011318" "870E",
+	.data_len = 23,
+	.locking_lang = GSM_DIALECT_TURKISH,
+	.single_lang = GSM_DIALECT_TURKISH,
+	.expected_text = {
+		0x0c, 0x5e, 0x7b, 0x7d, 0x5c, 0x5b, 0x7e, 0x5d, 0x7c, 0xc4,
+		0x9e, 0xc4, 0xb0, 0xc5, 0x9e, 0xc3, 0xa7, 0xe2, 0x82, 0xac,
+		0xc4, 0x9f, 0xc4, 0xb1, 0xc5, 0x9f, 0x00
+	}
+};
+
+static struct sms_charset_data sms_charset_portugal = {
+	.pdu =
+		"0001000B91" "5310101010" "1000008080" "8060402818" "0E888462C1"
+		"68381E9088" "6442A9582E" "988C06C4E9" "783EA09068" "442A994EA8"
+		"946AC56AB9" "5EB0986C46" "ABD96EB89C" "6EC7EBF97E" "C0A070482C"
+		"1A8FC8A472" "C96C3A9FD0" "A8744AAD5A" "AFD8AC76CB" "ED7ABFE0B0"
+		"784C2E9BCF" "E8B47ACD6E" "BBDFF0B87C" "4EAFDBEFF8" "BC7ECFEFFB"
+		"FF",
+	.data_len = 112,
+	.locking_lang = GSM_DIALECT_PORTUGUESE,
+	.expected_text = {
+		0x40, 0xc2, 0xa3, 0x24, 0xc2, 0xa5, 0xc3, 0xaa, 0xc3, 0xa9,
+		0xc3, 0xba, 0xc3, 0xad, 0xc3, 0xb3, 0xc3, 0xa7, 0x0a, 0xc3,
+		0x94, 0xc3, 0xb4, 0x0d, 0xc3, 0x81, 0xc3, 0xa1, 0xce, 0x94,
+		0x5f, 0xc2, 0xaa, 0xc3, 0x87, 0xc3, 0x80, 0xe2, 0x88, 0x9e,
+		0x5e, 0x5c, 0xe2, 0x82, 0xac, 0xc3, 0x93, 0x7c, 0x20, 0xc3,
+		0x82, 0xc3, 0xa2, 0xc3, 0x8a, 0xc3, 0x89, 0x20, 0x21, 0x22,
+		0x23, 0xc2, 0xba, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b,
+		0x2c, 0x2d, 0x2e, 0x2f, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35,
+		0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f,
+		0xc3, 0x8d, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48,
+		0x49, 0x4a, 0x4b, 0x4c, 0x4d, 0x4e, 0x4f, 0x50, 0x51, 0x52,
+		0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0xc3, 0x83,
+		0xc3, 0x95, 0xc3, 0x9a, 0xc3, 0x9c, 0xc2, 0xa7, 0x7e, 0x61,
+		0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6a, 0x6b,
+		0x6c, 0x6d, 0x6e, 0x6f, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75,
+		0x76, 0x77, 0x78, 0x79, 0x7a, 0xc3, 0xa3, 0xc3, 0xb5, 0x60,
+		0xc3, 0xbc, 0xc3, 0xa0, 0x00
+	}
+};
+
+static struct sms_charset_data sms_charset_portugal_ext = {
+	.pdu =
+		"0001000B91" "5310101010" "1000003184" "C446B16038" "1E1BC96662"
+		"D9543696CD" "6583D9643C" "1BD42675D9" "F0C01B9F86" "02CC74B75C"
+		"0EE68030EC" "F91D",
+	.data_len = 43,
+	.locking_lang = GSM_DIALECT_PORTUGUESE,
+	.single_lang = GSM_DIALECT_PORTUGUESE,
+	.expected_text = {
+		0xc3, 0xaa, 0xc3, 0xa7, 0x0c, 0xc3, 0x94, 0xc3, 0xb4, 0xc3,
+		0x81, 0xc3, 0xa1, 0xce, 0xa6, 0xce, 0x93, 0x5e, 0xce, 0xa9,
+		0xce, 0xa0, 0xce, 0xa8, 0xce, 0xa3, 0xce, 0x98, 0xc3, 0x8a,
+		0x7b, 0x7d, 0x5c, 0x5b, 0x7e, 0x5d, 0x7c, 0xc3, 0x80, 0xc3,
+		0x8d, 0xc3, 0x93, 0xc3, 0x9a, 0xc3, 0x83, 0xc3, 0x95, 0xc3,
+		0x82, 0xe2, 0x82, 0xac, 0xc3, 0xad, 0xc3, 0xb3, 0xc3, 0xba,
+		0xc3, 0xa3, 0xc3, 0xb5, 0xc3, 0xa2, 0x00
+	}
+};
+
+static struct sms_charset_data sms_charset_spain = {
+	.pdu =
+		"0001000B91" "5310101010" "100000269B" "C446B1A16C" "509BD4E6B5"
+		"E16D7A1BDF" "06B8096E92" "9BE7A6BA09" "6FCA9BF4E6" "BDA903",
+	.data_len = 34,
+	.locking_lang = GSM_DIALECT_SPANISH,
+	.single_lang = GSM_DIALECT_SPANISH,
+	.expected_text = {
+		0xc3, 0xa7, 0x0c, 0x5e, 0x7b, 0x7d, 0x5c, 0x5b, 0x7e, 0x5d,
+		0x7c, 0xc3, 0x81, 0xc3, 0x8d, 0xc3, 0x93, 0xc3, 0x9a, 0xc3,
+		0xa1, 0xe2, 0x82, 0xac, 0xc3, 0xad, 0xc3, 0xb3, 0xc3, 0xba,
+		0x00
+	}
+};
+
+static void test_sms_charset(gconstpointer param)
+{
+	gboolean ret;
+	struct sms sms;
+	unsigned char *pdu;
+	unsigned char *unpacked;
+	long pdu_len;
+	int data_len;
+	enum sms_charset sms_charset;
+	gboolean sms_compressed;
+	char *text;
+	struct sms_charset_data *data = (struct sms_charset_data *)param;
+
+	pdu = decode_hex(data->pdu, -1, &pdu_len, 0);
+
+	g_assert(pdu);
+	g_assert(pdu_len == (gint64)strlen(data->pdu) / 2);
+
+	ret = sms_decode(pdu, pdu_len, FALSE, pdu_len, &sms);
+
+	g_assert(ret);
+
+	g_free(pdu);
+
+	g_assert(sms.type == SMS_TYPE_DELIVER);
+
+	ret = sms_dcs_decode(sms.deliver.dcs, NULL, &sms_charset,
+				&sms_compressed, NULL);
+
+	g_assert(ret);
+	g_assert(sms_charset == SMS_CHARSET_7BIT);
+	g_assert(sms_compressed == FALSE);
+
+	data_len = sms_udl_in_bytes(sms.deliver.udl, sms.deliver.dcs);
+
+	g_assert(data_len == data->data_len);
+
+	unpacked = unpack_7bit(sms.deliver.ud, data_len, 0, FALSE,
+				sms.deliver.udl, NULL, 0xff);
+
+	g_assert(unpacked);
+
+	text = convert_gsm_to_utf8_with_lang(unpacked, -1, NULL, NULL, 0xff,
+		data->locking_lang, data->single_lang);
+
+	g_assert(text);
+
+	g_free(unpacked);
+
+	g_assert(strcmp(data->expected_text, text) == 0);
+
+	g_free(text);
+}
+
 struct text_format_header {
 	unsigned char len;
 	unsigned char start;
@@ -785,7 +998,7 @@ static void test_assembly()
 	if (g_test_verbose())
 		g_printf("Text:\n%s\n", utf8);
 
-	l = sms_text_prepare(utf8, ref, TRUE, NULL, FALSE);
+	l = sms_text_prepare("555", utf8, ref, TRUE, FALSE);
 	g_assert(l);
 	g_assert(g_slist_length(l) == 3);
 
@@ -815,7 +1028,8 @@ static void test_prepare_7bit()
 	int encoded_tpdu_len;
 	char *encoded_pdu;
 
-	r = sms_text_prepare(test_no_fragmentation_7bit, 0, FALSE, NULL, FALSE);
+	r = sms_text_prepare("555", test_no_fragmentation_7bit, 0,
+				FALSE, FALSE);
 
 	g_assert(r != NULL);
 
@@ -897,7 +1111,7 @@ static void test_prepare_concat(gconstpointer data)
 	if (g_test_verbose())
 		g_print("strlen: %zd\n", strlen(test->str));
 
-	r = sms_text_prepare(test->str, 0, TRUE, NULL, FALSE);
+	r = sms_text_prepare("+15554449999", test->str, 0, TRUE, FALSE);
 	g_assert(r);
 	g_assert(g_slist_length(r) == test->segments);
 
@@ -906,7 +1120,6 @@ static void test_prepare_concat(gconstpointer data)
 
 		sms = l->data;
 
-		sms_address_from_string(&sms->submit.daddr, "+15554449999");
 		sms_encode(sms, &pdu_len, &tpdu_len, pdu);
 		g_assert(pdu_len == (tpdu_len + 1));
 
@@ -973,7 +1186,7 @@ static void test_limit(gunichar uni, int target_size, gboolean use_16bit)
 
 	utf8[i] = '\0';
 
-	l = sms_text_prepare(utf8, 0, use_16bit, NULL, FALSE);
+	l = sms_text_prepare("555", utf8, 0, use_16bit, FALSE);
 
 	g_assert(l);
 	g_assert(g_slist_length(l) == 255);
@@ -986,7 +1199,7 @@ static void test_limit(gunichar uni, int target_size, gboolean use_16bit)
 	memcpy(utf8 + i, utf8_char, stride);
 	utf8[i+stride] = '\0';
 
-	l = sms_text_prepare(utf8, 0, use_16bit, NULL, FALSE);
+	l = sms_text_prepare("555", utf8, 0, use_16bit, FALSE);
 
 	g_assert(l == NULL);
 	g_free(utf8);
@@ -1273,16 +1486,22 @@ static void test_sr_assembly()
 				"940A00";
 	const char *sr_pdu2 = "06050D91945152991136F00160124130640A0160124130"
 				"450A00";
+	const char *sr_pdu3 = "0606098121436587F9019012413064A0019012413045A0"
+				"00";
         struct sms sr1;
 	struct sms sr2;
+	struct sms sr3;
 	unsigned char pdu[176];
 	long pdu_len;
 	struct status_report_assembly *sra;
 	gboolean delivered;
-	unsigned int id;
 	struct sms_address addr;
+	unsigned char sha1[SMS_MSGID_LEN] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
+						10, 11, 12, 13, 14, 15,
+						16, 17, 18, 19 };
+	unsigned char id[SMS_MSGID_LEN];
 
-	/* mr 4 & mr 5 */
+	/* international address, mr 4 & mr 5 */
 
         decode_hex_own_buf(sr_pdu1, -1, &pdu_len, 0, pdu);
 	g_assert(sms_decode(pdu, pdu_len, FALSE, 26, &sr1) == TRUE);
@@ -1290,25 +1509,72 @@ static void test_sr_assembly()
 	decode_hex_own_buf(sr_pdu2, -1, &pdu_len, 0, pdu);
 	g_assert(sms_decode(pdu, pdu_len, FALSE, 26, &sr2) == TRUE);
 
-	g_print("sr1 address: %s, mr: %d\n",
+	/* national address, mr 6 */
+
+	decode_hex_own_buf(sr_pdu3, -1, &pdu_len, 0, pdu);
+	g_assert(sms_decode(pdu, pdu_len, FALSE, 24, &sr3) == TRUE);
+
+	if (g_test_verbose()) {
+		g_print("sr1 address: %s, mr: %d\n",
 			sms_address_to_string(&sr1.status_report.raddr),
 			sr1.status_report.mr);
 
-	g_print("sr2 address: %s, mr: %d\n",
+		g_print("sr2 address: %s, mr: %d\n",
 			sms_address_to_string(&sr2.status_report.raddr),
 			sr2.status_report.mr);
 
+		g_print("sr3 address: %s, mr: %d\n",
+			sms_address_to_string(&sr3.status_report.raddr),
+			sr3.status_report.mr);
+	}
+
 	sms_address_from_string(&addr, "+4915259911630");
 
 	sra = status_report_assembly_new(NULL);
-	status_report_assembly_add_fragment(sra, 42, &addr, 4, time(NULL), 2);
-	status_report_assembly_add_fragment(sra, 42, &addr, 5, time(NULL), 2);
 
-	g_assert(!status_report_assembly_report(sra, &sr1, &id, &delivered));
-	g_assert(status_report_assembly_report(sra, &sr2, &id, &delivered));
+	status_report_assembly_add_fragment(sra, sha1, &addr, 4, time(NULL), 2);
+	status_report_assembly_add_fragment(sra, sha1, &addr, 5, time(NULL), 2);
 
-	g_assert(id == 42);
+	status_report_assembly_expire(sra, time(NULL) + 40);
+	g_assert(g_hash_table_size(sra->assembly_table) == 0);
+
+	status_report_assembly_add_fragment(sra, sha1, &addr, 4, time(NULL), 2);
+	status_report_assembly_add_fragment(sra, sha1, &addr, 5, time(NULL), 2);
+
+	g_assert(!status_report_assembly_report(sra, &sr1, id, &delivered));
+	g_assert(status_report_assembly_report(sra, &sr2, id, &delivered));
+
+	g_assert(memcmp(id, sha1, SMS_MSGID_LEN) == 0);
 	g_assert(delivered == TRUE);
+
+	/*
+	 * Send sms-message in the national address-format,
+	 * but receive in the international address-format.
+	 */
+	sms_address_from_string(&addr, "9911630");
+	status_report_assembly_add_fragment(sra, sha1, &addr, 4, time(NULL), 2);
+	status_report_assembly_add_fragment(sra, sha1, &addr, 5, time(NULL), 2);
+
+	g_assert(!status_report_assembly_report(sra, &sr1, id, &delivered));
+	g_assert(status_report_assembly_report(sra, &sr2, id, &delivered));
+
+	g_assert(memcmp(id, sha1, SMS_MSGID_LEN) == 0);
+	g_assert(delivered == TRUE);
+	g_assert(g_hash_table_size(sra->assembly_table) == 0);
+
+	/*
+	 * Send sms-message in the international address-format,
+	 * but receive in the national address-format.
+	 */
+	sms_address_from_string(&addr, "+358123456789");
+	status_report_assembly_add_fragment(sra, sha1, &addr, 6, time(NULL), 1);
+
+	g_assert(status_report_assembly_report(sra, &sr3, id, &delivered));
+
+	g_assert(memcmp(id, sha1, SMS_MSGID_LEN) == 0);
+	g_assert(delivered == TRUE);
+	g_assert(g_hash_table_size(sra->assembly_table) == 0);
+
 	status_report_assembly_free(sra);
 }
 
@@ -1408,6 +1674,34 @@ int main(int argc, char **argv)
 	g_test_add_func("/testsms/Test Simple Submit", test_simple_submit);
 	g_test_add_func("/testsms/Test Submit Encode", test_submit_encode);
 
+	g_test_add_data_func("/testsms/Test "
+		"GSM 7 bit Default Alphabet Decode",
+		&sms_charset_default, test_sms_charset);
+
+	g_test_add_data_func("/testsms/Test "
+		"GSM 7 bit Default Alphabet Extension Table Decode",
+		&sms_charset_default_ext, test_sms_charset);
+
+	g_test_add_data_func("/testsms/Test "
+		"Turkish National Language Locking Shift Table Decode",
+		&sms_charset_turkey, test_sms_charset);
+
+	g_test_add_data_func("/testsms/Test "
+		"Turkish National Language Single Shift Table Decode",
+		&sms_charset_turkey_ext, test_sms_charset);
+
+	g_test_add_data_func("/testsms/Test "
+		"Portuguese National Language Locking Shift Table Decode",
+		&sms_charset_portugal, test_sms_charset);
+
+	g_test_add_data_func("/testsms/Test "
+		"Portuguese National Language Single Shift Table Decode",
+		&sms_charset_portugal_ext, test_sms_charset);
+
+	g_test_add_data_func("/testsms/Test "
+		"Spanish National Language Single Shift Table Decode",
+		&sms_charset_spain, test_sms_charset);
+
 	g_test_add_data_func("/testsms/Test EMS UDH 1",
 			&ems_udh_test_1, test_ems_udh);
 	g_test_add_data_func("/testsms/Test EMS UDH 2",
